package info.bukova.isspst.services.requirement;

import info.bukova.isspst.Constants;
import info.bukova.isspst.data.NumberSeries;
import info.bukova.isspst.data.RequirementState;
import info.bukova.isspst.data.TripBill;
import info.bukova.isspst.data.TripRequirement;
import info.bukova.isspst.services.LazyLoader;
import info.bukova.isspst.services.tripbill.TripBillService;
import info.bukova.isspst.services.workgroups.WorkgroupService;

import java.util.Date;

import org.hibernate.LazyInitializationException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;

public class TripRequirementServiceImpl extends RequirementBaseServiceImpl<TripRequirement>
		implements TripRequirementService, RequirementBaseService<TripRequirement> {
	
	@Autowired
	private RequirementTypeService reqTypeService;
	@Autowired
	private WorkgroupService workgroupService;
	@Autowired
	private TripBillService tripBillService;

	@Override
	protected TripRequirement createEntity() {
		TripRequirement tr = new TripRequirement();
		tr.setReqDate(new Date());
		tr.setType(reqTypeService.getTypeById(Constants.REQTYPE_BUSINESSTRIP));
		tr.setState(RequirementState.NEW);

		return tr;
	}

	@Override
	@Transactional
	@LazyLoader("form")
	public void loadPassangers(TripRequirement entity) {
		try {
			if (entity.getPassengers() == null) {
				throw new LazyInitializationException("");
			}
			
			entity.getPassengers().size();
		} catch (LazyInitializationException ex) {
			TripRequirement e = dao.getById(entity.getId());
			e.getPassengers().size();
			entity.setPassengers(e.getPassengers());	
		}
	}

	@Override
	@Transactional
	@PreAuthorize("this.canApprove(#entity)")
	public void approve(TripRequirement entity) {
		super.approve(entity);
		
		if (entity.getState() == RequirementState.APPROVED) {
			TripBill bill = tripBillService.createTripBill(entity);
			tripBillService.add(bill);
			bill.setOwnedBy(entity.getOwnedBy());
			tripBillService.update(bill);
		}
	}

	@Override
	protected String getNumberSerie() {
		String currentNumber = "";
		
		if (getNumberSeriesService() != null)
		{
			NumberSeries ns = getNumberSeriesService().getNumberSerie(Constants.MOD_REQUIREMENTS);
			
			if (ns != null)
			{
				currentNumber = ns.getCurrentNumber();
				getNumberSeriesService().increase(ns);
			}
		}
		
		return currentNumber;
	}

}
